/**
 * @autor Marcos Antonio Pereira Silva
 */

package br.urca.solicitacoes.ejb;

import java.util.Set;

import br.urca.solicitacoes.Funcionario;
import br.urca.solicitacoes.SolicitacoesException;
import br.urca.util.UtilsString;

final class FiltroSituacoesAtivasFuncionario
{
	private FiltroSituacoesAtivasFuncionario()
	{
	}

	public static String filtrar(String jpql)
	{
		return filtrar(jpql, "f");
	}

	public static String filtrar(String jpql, String variavelUsuario)
	{
		validar(Funcionario.SITUACOES_ATIVAS, jpql, variavelUsuario);

		int indiceFinalClausulaWHERE = obterIndiceFinalClausulaWHERE(jpql);
		String filtro = String.format(
			" %1$s (%2$s.situacao IN (%3$s) OR %2$s.situacao IS NULL) ",
			clausulaWHEREExistente(jpql) ? "AND" : "WHERE",
			variavelUsuario,
			obterSituacoesAtivasComoString(Funcionario.SITUACOES_ATIVAS));

		return UtilsString.inserir(jpql, indiceFinalClausulaWHERE, 0, filtro);
	}

	private static void validar(
		Set<String> situacoesAtivas, String jpql, String variavelUsuario)
	{
		if (situacoesAtivas == null)
		{
			throw new NullPointerException("Situaes ativas nula.");
		}
		if (situacoesAtivas.isEmpty())
		{
			throw new IllegalArgumentException(
				"Situaes ativas inexistentes.");
		}
		if (UtilsString.branco(jpql))
		{
			throw new IllegalArgumentException("JPQL nula ou em branco.");
		}
		if (UtilsString.branco(variavelUsuario))
		{
			throw new IllegalArgumentException(
				"Varivel do usurio nula ou em branco.");
		}
	}

	private static int obterIndiceFinalClausulaWHERE(String jpql)
	{
		int indice = obterIndiceInicioClausulaJPQL(jpql, ClausulaJPQL.GROUP_BY);
		if (indice >= 0)
		{
			return indice;
		}

		indice = obterIndiceInicioClausulaJPQL(jpql, ClausulaJPQL.ORDER_BY);
		if (indice >= 0)
		{
			return indice;
		}

		indice = jpql.length();
		return indice;
	}

	private static boolean clausulaWHEREExistente(String jpql)
	{
		return obterIndiceInicioClausulaJPQL(jpql, ClausulaJPQL.WHERE) >= 0;
	}

	private static int obterIndiceInicioClausulaJPQL(
		String jpql, ClausulaJPQL clausula)
	{
		StringBuilder palavra = new StringBuilder();

		boolean aspas = false;
		char aspasAbertura = 0;
		boolean aspasBalanceadas = false;

		boolean parenteses = false;
		int totalParenteses = 0;

		int i = 0;
		while (i < jpql.length())
		{
			char c = jpql.charAt(i);
			if (c == '\'' || c == '"')
			{
				if (aspas)
				{
					if (c == aspasAbertura)
					{
						aspasBalanceadas = !aspasBalanceadas;
						if (aspasBalanceadas && (i == jpql.length() - 1 ||
							jpql.charAt(i + 1) != aspasAbertura))
						{
							aspas = false;
						}
					}
				}
				else
				{
					if (c == '"' && !parenteses &&
						clausula.inicio(palavra.toString()))
					{
						return clausula.obterIndiceInicio(i);
					}
					aspas = true;
					aspasAbertura = c;
					aspasBalanceadas = false;
					palavra.setLength(0);
				}
			}
			else if (c == '(' || c == ')')
			{
				if (!aspas)
				{
					if (c == '(')
					{
						if (totalParenteses == 0)
						{
							if (clausula.inicio(palavra.toString()))
							{
								return clausula.obterIndiceInicio(i);
							}
						}
						totalParenteses++;
						palavra.setLength(0);
					}
					else
					{
						totalParenteses--;
						if (totalParenteses < 0)
						{
							throw new SolicitacoesException(
								"Falta parnteses de abertura na JPQL.");
						}
					}
					parenteses = totalParenteses > 0;
				}
			}
			else if (!aspas && !parenteses)
			{
				if (Character.isLetter(c))
				{
					palavra.append(c);
				}
				else if (Character.isWhitespace(c))
				{
					if (clausula.inicio(palavra.toString()))
					{
						return clausula.obterIndiceInicio(i);
					}
					palavra.setLength(0);
				}
			}
			i++;
		}
		if (aspas)
		{
			throw new SolicitacoesException("Falta fechar aspas na JPQL.");
		}
		if (parenteses)
		{
			throw new SolicitacoesException(
				"Falta parnteses de fechamento na JPQL.");
		}
		return -1;
	}

	private static String obterSituacoesAtivasComoString(
		Set<String> situacoesAtivas)
	{
		StringBuilder situacoesAtivasString =
			new StringBuilder(4 * situacoesAtivas.size() - 1);
		for (String situacaoAtiva : situacoesAtivas)
		{
			if (situacoesAtivasString.length() > 0)
			{
				situacoesAtivasString.append(',');
			}
			situacoesAtivasString.append('\'');
			situacoesAtivasString.append(situacaoAtiva);
			situacoesAtivasString.append('\'');
		}
		return situacoesAtivasString.toString();
	}

	private static enum ClausulaJPQL
	{
		WHERE ("WHERE"),
		GROUP_BY ("GROUP"),
		ORDER_BY ("ORDER");

		private final String fPrimeiraPalavra;

		private ClausulaJPQL(String primeiraPalavra)
		{
			fPrimeiraPalavra = primeiraPalavra;
		}

		private boolean inicio(String palavra)
		{
			return palavra.equalsIgnoreCase(fPrimeiraPalavra);
		}

		private int obterIndiceInicio(int indiceAtual)
		{
			return indiceAtual - fPrimeiraPalavra.length();
		}
	}
}
