/**
 * @autor Marcos Antonio Pereira Silva
 */

package br.urca.solicitacoes.ejb;

import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.apache.commons.validator.EmailValidator;

import br.urca.seguranca.codificacao.UtilsSenha;
import br.urca.solicitacoes.AdministracaoFuncionario;
import br.urca.solicitacoes.DadosAtualizacaoFuncionarioAutenticado;
import br.urca.solicitacoes.DadosSenha;
import br.urca.solicitacoes.EmailInvalidoException;
import br.urca.solicitacoes.Funcionario;
import br.urca.solicitacoes.IGerenciadorFuncionarioAutenticado;
import br.urca.solicitacoes.PermissaoFuncionario;
import br.urca.solicitacoes.SenhaException;
import br.urca.solicitacoes.Setor;
import br.urca.solicitacoes.SolicitacoesException;
import br.urca.util.UtilsString;

@Stateless
public class GerenciadorFuncionarioAutenticadoBean
	implements IGerenciadorFuncionarioAutenticado
{
	@EJB
	private ILocalizadorFuncionario fLocalizadorFuncionario;

	public void atualizarFuncionario(DadosAtualizacaoFuncionarioAutenticado
		dadosAtualizacaoFuncionarioAutenticado)
	{
		if (dadosAtualizacaoFuncionarioAutenticado == null)
		{
			throw new NullPointerException(
				"Dados de atualizao do funcionrio nulo.");
		}
		if (dadosAtualizacaoFuncionarioAutenticado.getEmail() != null)
		{
			EmailValidator validadorEmail = EmailValidator.getInstance();
			if (!validadorEmail.isValid(
				dadosAtualizacaoFuncionarioAutenticado.getEmail()))
			{
				throw new EmailInvalidoException("E-mail invlido.");
			}
		}
		if (dadosAtualizacaoFuncionarioAutenticado.getSetorPadrao() == null)
		{
			throw new NullPointerException("Setor padro nulo.");
		}
		
		Funcionario funcionario =
			fLocalizadorFuncionario.localizarFuncionarioAutenticado();
		
		if (funcionario.getSetor() == null)
		{
			throw new SolicitacoesException("Setor nulo.");
		}

		funcionario.setEmail(dadosAtualizacaoFuncionarioAutenticado.getEmail());

		if (setorPadraoAlterado(
			dadosAtualizacaoFuncionarioAutenticado.getSetorPadrao(),
			funcionario))
		{
			if (setorPadraoInvalido(
				dadosAtualizacaoFuncionarioAutenticado.getSetorPadrao(),
				funcionario))
			{
				throw new SolicitacoesException("Setor padro inexistente.");
			}
			
			if (dadosAtualizacaoFuncionarioAutenticado.getSetorPadrao().equals(
				funcionario.getSetor()))
			{
				funcionario.setSetorSolicitacoes(null);
			}
			else
			{
				funcionario.setSetorSolicitacoes(
					dadosAtualizacaoFuncionarioAutenticado.getSetorPadrao());
			}
		}
		
		if (dadosAtualizacaoFuncionarioAutenticado.getDadosSenha() != null)
		{
			atualizarFuncionario(
				dadosAtualizacaoFuncionarioAutenticado.getDadosSenha());
		}
	}

	public void atualizarFuncionario(DadosSenha dadosSenha)
	{
		if (dadosSenha == null || dadosSenha.getSenhaAtual() == null ||
			dadosSenha.getNovaSenha() == null ||
			dadosSenha.getConfirmacaoNovaSenha() == null)
		{
			throw new NullPointerException("Dado(s) nulo(s).");
		}
		
		Funcionario funcionario =
			fLocalizadorFuncionario.localizarFuncionarioAutenticado();
		
		// Validar senha atual
		if (UtilsString.branco(dadosSenha.getSenhaAtual()))
		{
			throw new SenhaException("Senha atual em branco.");
		}
		if (!UtilsSenha.senhasIguais(dadosSenha.getSenhaAtual(),
			funcionario.getSenha(), funcionario.getHashSalt()))
		{
			throw new SenhaException("Senha atual invlida.");
		}

		// Validar nova senha
		validarNovaSenha(dadosSenha.getNovaSenha());
		
		// Validar confirmao da nova senha
		if (!dadosSenha.getConfirmacaoNovaSenha().equals(
			dadosSenha.getNovaSenha()))
		{
			throw new SenhaException(
				"Confirmao da nova senha diferente da nova senha.");
		}
		
		// Validar senha provisria
		boolean senhaProvisoria = !funcionario.isSenhaAlterada();
		if (senhaProvisoria &&
			dadosSenha.getNovaSenha().equals(dadosSenha.getSenhaAtual()))
		{
			throw new SenhaException(
				"A nova senha no pode ser igual  senha atual.");
		}

		// Atualizar
		String hashSalt = UtilsSenha.gerarHashSalt();
		String novaSenhaCodificada =
			UtilsSenha.codificarSenha(dadosSenha.getNovaSenha(), hashSalt);
		funcionario.setSenha(novaSenhaCodificada);
		funcionario.setHashSalt(hashSalt);
		funcionario.setSenhaAlterada(true);		
	}

	private boolean setorPadraoAlterado(
		Setor setorPadrao, Funcionario funcionario)
	{
		if (funcionario.getSetorSolicitacoes() == null)
		{
			return !setorPadrao.equals(funcionario.getSetor());
		}
		else
		{
			return !setorPadrao.equals(funcionario.getSetorSolicitacoes());
		}
	}

	private boolean setorPadraoInvalido(
		Setor setorPadrao, Funcionario funcionario)
	{
		return !funcionario.getSetor().equals(setorPadrao) &&
			funcionarioNaoPossuiPermissoesNoSetorPadrao(
			funcionario, setorPadrao);
	}
	
	private boolean funcionarioNaoPossuiPermissoesNoSetorPadrao(
		Funcionario funcionario, Setor setorPadrao)
	{
		for (AdministracaoFuncionario administracao :
			funcionario.getAdministracoes())
		{
			if (administracao.getSetor().equals(setorPadrao))
			{
				return false;
			}
		}
		
		for (PermissaoFuncionario permissao : funcionario.getPermissoes())
		{
			if (permissao.getSetor().equals(setorPadrao))
			{
				return false;
			}
		}
		
		return true;
	}

	private void validarNovaSenha(String novaSenha) throws SenhaException
	{
		final int TAMANHO_MINIMO_SENHA = 6;
		final int TAMANHO_MAXIMO_SENHA = 30;

		// Branco
		if (UtilsString.branco(novaSenha))
		{
			throw new SenhaException("Nova senha em branco.");
		}

		// Tamanho
		if (novaSenha.length() < TAMANHO_MINIMO_SENHA ||
			novaSenha.length() > TAMANHO_MAXIMO_SENHA)
		{
			throw new SenhaException("Tamanho da nova senha invlido. " +
				"A nova senha deve conter no mnimo " +
				TAMANHO_MINIMO_SENHA + " e no mximo " +
				TAMANHO_MAXIMO_SENHA + " caracteres.");
		}

		// Espaos em branco
		if (novaSenha.contains(" "))
		{
			throw new SenhaException(
				"A nova senha no pode conter espaos em branco.");
		}

		// Letras e nmeros
		boolean senhaComLetras = false;
		boolean senhaComNumeros = false;
		for (int i = 0; i < novaSenha.length(); i++)
		{
			if (Character.isLetter(novaSenha.charAt(i)))
			{
				senhaComLetras = true;
				if (senhaComNumeros)
				{
					break;
				}
			}
			else if (Character.isDigit(novaSenha.charAt(i)))
			{
				senhaComNumeros = true;
				if (senhaComLetras)
				{
					break;
				}
			}			
		}
		boolean senhaComLetrasENumeros = senhaComLetras && senhaComNumeros;
		if (!senhaComLetrasENumeros)
		{
			throw new SenhaException("A nova senha deve conter letras e " +
				"nmeros. Outros caracteres tambm so permitidos, " +
				"mas no so obrigatrios.");
		}
	}
}
